#version 330
#extension GL_ARB_separate_shader_objects : enable





layout(location = 0) smooth in vec2 _fin_fragTexCoord;

struct SFragmentInput
{
	vec2 fragTexCoord;
	bool IsFrontFace;
};

layout(location = 0) out vec4 _fout_outColor;

struct SFragmentOutput
{
	vec4 outColor;
};

layout( std140) uniform USceneInfo
{
	mat4 ViewProj;
	mat4 View;
	mat4 Proj;
	mat4 InvView;
	mat4 InvViewProj;
	mat4 UserToLHZ;
	mat4 LHZToUser;
	mat4 BillboardingView;
	mat4 PackNormalView;
	mat4 UnpackNormalView;
	vec4 SideVector;
	vec4 DepthVector;
	vec2 ZBufferLimits;
	float Handedness;
} SceneInfo;
layout( std140) uniform ULightInfo
{
	vec4 Transform;
	vec4 Color;
	vec4 Ambient;
	float Radius;
} LightInfo;
 uniform sampler2D BRDFLUTSampler;
 uniform samplerCube EnvironmentMapSampler;
 uniform sampler2D DepthSampler;
 uniform sampler2D NormalRoughMetalSampler;
 uniform sampler2D DiffuseSampler;



vec3 DepthToWorldPos(float pDepthValue, vec2 pUv, mat4 pInvViewProj)
{
	vec2 clipSpace = pUv * 2.0 - 1.0;
	vec4 projPos = vec4(clipSpace, pDepthValue, 1.0);
	projPos =  ((pInvViewProj) * (projPos)) ;
	return projPos.xyz / projPos.w;
}

vec3 UnpackNormalSpheremap(vec2 normal  )
{
	vec2 fenc = normal * 4.0f - 2.0f;
	float f = dot(fenc, fenc);
	float g = sqrt(1.0f - f / 4.0f);
	vec3 outNormal;
	outNormal.xy = fenc * g;
	outNormal.z = 1.0f - f / 2.0f;
	return  (( SceneInfo.UnpackNormalView ) * (vec4(outNormal, 0.0f))) .xyz;
}

vec3 UnpackNormalStd(vec3 normal)
{
	return normal * 2.0 - 1.0;
}

float getColorLuminance(vec3 color)
{
	return 0.299 * color.x + 0.587 * color.y + 0.114 * color.z;
}







float unlinearizeFallOff(float x)
{
	float k = 1.0f / 0.5f;
	float A = (1.0f) - (1.0f / ((k + 1.0f) * (k + 1.0f)));
	float B = (1.0f / ((k + 1.0f) * (k + 1.0f)));
	float C = (1.0f / ((x * k + 1.0f) * (x * k + 1.0f)));
	return (C - B) / A;
}

void FragmentMain( in SFragmentInput  fInput,  out SFragmentOutput  fOutput  )
{
	vec3 lightPosition =  LightInfo.Transform .xyz;
	vec3 lightColor =  LightInfo.Color .xyz;
	float lightRadius =  LightInfo.Radius ;
	mat4 invViewProj =  SceneInfo.InvViewProj ;
	vec3 viewPosition =  SceneInfo.BillboardingView [3] .xyz;

	vec3 surfaceColor =  texture(DiffuseSampler, fInput.fragTexCoord) .rgb;
	float depthValue =  texture(DepthSampler, fInput.fragTexCoord) .r;
	vec3 worldPos = DepthToWorldPos(depthValue, fInput.fragTexCoord, invViewProj);
	vec4 packedNormalSpec =  texture(NormalRoughMetalSampler, fInput.fragTexCoord) ;
	vec3 normal = UnpackNormalSpheremap(packedNormalSpec.rg  );


	vec3 worldPosToLight = lightPosition - worldPos;
	vec3 worldPosToLightDirection = normalize(worldPosToLight);
	float diffuseIntensity = max(0., dot(worldPosToLightDirection, normal));

	float normalizedDistanceToLight = min(1.0f, length(worldPosToLight) / lightRadius);
	float lightAttenuation = unlinearizeFallOff(normalizedDistanceToLight);

	float specularIntensity = 0.;

	if ( any( notEqual(packedNormalSpec.rg, vec2(0,0)) ) )
	{
		vec3 eyeToPixelDirection = normalize(worldPos - viewPosition);
		vec3 reflectionDirection = reflect(eyeToPixelDirection, normal);
		specularIntensity = max(0., dot(reflectionDirection, worldPosToLightDirection));
	}

	if ( any( notEqual(packedNormalSpec.ab, vec2(0,0)) )  || specularIntensity != 0.)
	{
		float glossiness = packedNormalSpec.a;
		float specStrength = packedNormalSpec.b;
		specularIntensity = pow(specularIntensity, glossiness) * specStrength;
	}

	vec3 diffuse = surfaceColor * lightColor * diffuseIntensity;
	vec3 specular = surfaceColor * lightColor * specularIntensity;

	vec3 finalColor = (diffuse + specular) * lightAttenuation;

	fOutput.outColor = vec4(finalColor, 0.0f);
}


void main()
{
	SFragmentInput fInput;
	SFragmentOutput fOutput;

	fInput.fragTexCoord = _fin_fragTexCoord;
	fInput.IsFrontFace = gl_FrontFacing;
	FragmentMain(fInput, fOutput);
	_fout_outColor = fOutput.outColor;
}
