#version 330
#extension GL_ARB_separate_shader_objects : enable





layout(location = 0) smooth in vec2 _fin_fragTexCoord;

struct SFragmentInput
{
	vec2 fragTexCoord;
	bool IsFrontFace;
};

layout(location = 0) out vec4 _fout_outColor;

struct SFragmentOutput
{
	vec4 outColor;
};

 uniform sampler2D Texture;
 uniform sampler2D LookUp;

layout( std140) uniform UBlurInfo
{
	vec2 ContextSize;
	vec2 Direction;
} BlurInfo;


vec4 blur5(vec2 uv, float radius  )
{
	vec2 direction =  BlurInfo.Direction  * radius;
	vec2 resolution =  BlurInfo.ContextSize ;
	vec4 color =  vec4(0,0,0,0) ;
	vec2 off1 = (1.3333333333333333 * direction) / resolution;
	color +=  texture(Texture, uv)  * 0.29411764705882354;
	color +=  texture(Texture, uv + off1)  * 0.35294117647058826;
	color +=  texture(Texture, uv - off1)  * 0.35294117647058826;
	return color;
}

vec4 blur9(vec2 uv, float radius  )
{
	vec2 direction =  BlurInfo.Direction  * radius;
	vec2 resolution =  BlurInfo.ContextSize ;
	vec4 color =  vec4(0,0,0,0) ;
	vec2 off1 = (1.3846153846 * direction) / resolution;
	vec2 off2 = (3.2307692308 * direction) / resolution;
	color +=  texture(Texture, uv)  * 0.2270270270;
	color +=  texture(Texture, uv + off1)  * 0.3162162162;
	color +=  texture(Texture, uv - off1)  * 0.3162162162;
	color +=  texture(Texture, uv + off2)  * 0.0702702703;
	color +=  texture(Texture, uv - off2)  * 0.0702702703;
	return color;
}

vec4 blur13(vec2 uv, float radius  )
{
	vec2 direction =  BlurInfo.Direction  * radius;
	vec2 resolution =  BlurInfo.ContextSize ;
	vec4 color =  vec4(0,0,0,0) ;
	vec2 off1 = (1.411764705882353 * direction) / resolution;
	vec2 off2 = (3.2941176470588234 * direction) / resolution;
	vec2 off3 = (5.176470588235294 * direction) / resolution;
	color +=  texture(Texture, uv)  * 0.1964825501511404;
	color +=  texture(Texture, uv + off1)  * 0.2969069646728344;
	color +=  texture(Texture, uv - off1)  * 0.2969069646728344;
	color +=  texture(Texture, uv + off2)  * 0.09447039785044732;
	color +=  texture(Texture, uv - off2)  * 0.09447039785044732;
	color +=  texture(Texture, uv + off3)  * 0.010381362401148057;
	color +=  texture(Texture, uv - off3)  * 0.010381362401148057;
	return color;
}

void FragmentMain( in SFragmentInput  fInput,  out SFragmentOutput  fOutput  )
{
	float blurRadius = 1.0f;



	blurRadius = min( texture(LookUp, fInput.fragTexCoord) .z, 1.0f);









	fOutput.outColor = blur5(fInput.fragTexCoord, blurRadius  );
}


void main()
{
	SFragmentInput fInput;
	SFragmentOutput fOutput;

	fInput.fragTexCoord = _fin_fragTexCoord;
	fInput.IsFrontFace = gl_FrontFacing;
	FragmentMain(fInput, fOutput);
	_fout_outColor = fOutput.outColor;
}
